#include "blurwaylandinterface.h"


static void handle_global(void *data,struct wl_registry *wl_registry,uint32_t name, const char *interface,  uint32_t version)
{
    auto blurInterface = reinterpret_cast<BlurWaylandInterface *>(data);
    blurInterface->handleGlobal(data,wl_registry,name,interface,version);
}
static void handle_global_remove(void *data, struct wl_registry *wl_registry, uint32_t name)
{
}

BlurWaylandInterface::BlurWaylandInterface(QObject *parent)
    : BlurAbstractInterface(parent)
{
    m_connection = KWayland::Client::ConnectionThread::fromApplication(qApp);
    m_registry = new Registry(this);
    m_registry->create(m_connection->display());

    connect(m_registry, &KWayland::Client::Registry::compositorAnnounced, this, [=](){
        const auto interface = m_registry->interface(KWayland::Client::Registry::Interface::Compositor);
        if (interface.name != 0) {
            m_compositor = m_registry->createCompositor(interface.name, interface.version, this);
        }
    });
    struct wl_registry_listener registry_listener = {
        .global = handle_global,
        .global_remove = handle_global_remove
    };
    wl_registry* registry = *m_registry;
    wl_registry_add_listener(registry, &registry_listener, this);

    m_registry->setup();
    m_connection->roundtrip();
}

void BlurWaylandInterface::handleGlobal(void *data, wl_registry *registry, uint32_t name, const char *interface, uint32_t version)
{
    if (strcmp(interface, ukui_blur_manager_interface.name) == 0)
    {
        ukui_blur_manager *blur_manager_handle =  (ukui_blur_manager *) wl_registry_bind(registry, name, &ukui_blur_manager_interface, version);
        if(blur_manager_handle)
        {
            m_blur_manager = new UkuiBlurManager(this);
            m_blur_manager->setup(blur_manager_handle);
        }
    }
}

bool BlurWaylandInterface::eventFilter(QObject *obj, QEvent *ev)
{
    auto window = qobject_cast<QWindow*>(obj);
    if(window && ev->type() == QEvent::Hide)
    {
        if(m_surfaces.contains(window))
        {
            auto surface = m_surfaces.value(window);
            if(surface)
            {
                surface->release();
                surface->destroy();
            }
            m_surfaces.remove(window);
        }
        if(m_regions.contains(window))
        {
            auto region = m_regions.value(window);
            if(region)
            {
                region->release();
                region->destroy();
            }
            m_regions.remove(window);
        }
        if(m_blurs.contains(window))
        {
            auto blur_handle = m_blurs.value(window);
            if(blur_handle)
            {
                blur_handle->release();
                blur_handle->destroy();
            }
            m_blurs.remove(window);
        }
    }
    return QObject::eventFilter(obj,ev);
}

bool BlurWaylandInterface::setBlurBehindWithStrength(QWindow *window, bool enable, const QRegion &region, uint32_t strength)
{
    if(!window || !m_blur_manager || !m_compositor)
        return false;

    auto surface = KWayland::Client::Surface::fromWindow(window);
    if (!surface)
        return false;
    if(!m_surfaces.contains(window))
    {
        m_surfaces.insert(window,surface);
    }
    if(enable)
    {
        UkuiBlur * blurHandle = m_blur_manager->createBlur(surface, this);
        if(!blurHandle)
            return false;
        if(!m_blurs.contains(window))
        {
            m_blurs.insert(window,blurHandle);
        }
        Region* region_hanlde = m_compositor->createRegion(this);
        if(!region_hanlde)
            return false;
        if(!m_regions.contains(window))
        {
            m_regions.insert(window,region_hanlde);
        }
        for(const QRect &rect : region)
        {
            region_hanlde->add(rect);
        }
        blurHandle->setRegion(region_hanlde);
        blurHandle->setStrength(strength);
        blurHandle->commit();
        window->installEventFilter(this);
        return true;
    }
    else
    {
        m_blur_manager->removeBlur(surface);
        window->removeEventFilter(this);
        return true;
    }

}
